<?php
/*
* Plugin Name: LivIcons Evolution
* Plugin URI: https://livicons.com
* Description: WordPress plugin with the next modern generation of animated SVG (vector) icons with individual animation for each one.
* Version: 2.8.379
* JavaScript main assets version: 2.8.XXX
* Author: DeeThemes
* Author URI: https://1.envato.market/c/1301577/275988/4415?u=https%3A%2F%2Fcodecanyon.net%2Fuser%2Fdeethemes
* Text Domain: livicons-evo
* Domain Path: /languages
* Copyright © 2019 | DeeThemes | https://livicons.com
*/
/*
* prefixes used for the plugin: livicons_evolution_ and LIVICONS_EVOLUTION_
*/

// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// plugin's constants
define( 'LIVICONS_EVOLUTION_VERSION', '2.8.379' );
define( 'LIVICONS_EVOLUTION_SLUG', 'livicons-evolution' );
define( 'LIVICONS_EVOLUTION_FILE', __FILE__ );
define( 'LIVICONS_EVOLUTION_PATH', plugin_dir_path(__FILE__) );
define( 'LIVICONS_EVOLUTION_URL', plugin_dir_url( __FILE__ ) );
define( 'LIVICONS_EVOLUTION_PLUGIN_SLUG', plugin_basename( __FILE__ ) );
define( 'LIVICONS_EVOLUTION_THIS_HOST', parse_url( get_site_url() , PHP_URL_HOST) );
define( 'LIVICONS_EVOLUTION_UPDATE_URL', 'https://deethemes.com/updates/update-livicons.php' );

// check for mandatory params
function livicons_evolution_check_mandatory_params() {
	// require minimum version of WordPress
	global $wp_version;
	if ( version_compare( $wp_version, '4.9', '<' ) ) {
		if( is_plugin_active( LIVICONS_EVOLUTION_PLUGIN_SLUG ) ) {
			deactivate_plugins( LIVICONS_EVOLUTION_PLUGIN_SLUG );
			wp_die( 
				sprintf(
					'<strong>' .
					esc_html__( 'LivIcons Evolution plugin', 'livicons-evo' ) . 
					'</strong> ' .
					esc_html__( 'requires WordPress 4.9 or higher, and has been deactivated! Please upgrade WordPress and try again.', 'livicons-evo') . 
					'<br><br>' .
					esc_html__( 'Back to', 'livicons-evo' ) .
					' <a href="%1$s">' .
					esc_html__( 'WordPress Dashboard', 'livicons-evo' ) .
					'</a>.',
					esc_url( admin_url() )
				)
			);
		}
	}
	// check whether the WordPress file system API can be used
	if( 'direct' === get_filesystem_method() ) {
		$creds = request_filesystem_credentials( content_url() . '/', '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			return false;
		}
	} else {
		// don't have direct write access.
		if( is_plugin_active( LIVICONS_EVOLUTION_PLUGIN_SLUG ) ) {
			deactivate_plugins( LIVICONS_EVOLUTION_PLUGIN_SLUG );
			wp_die( 
				sprintf(
					'<strong> ' .
					esc_html__( 'LivIcons Evolution plugin', 'livicons-evo' ) . 
					'</strong> ' .
					esc_html__( "requires the direct file system access on your server, and has been deactivated! Unfortunately, the plugin cannot be used. Please contact the plugin's author.", "livicons-evo") . 
					'<br><br>' .
					esc_html__( 'Back to', 'livicons-evo' ) .
					' <a href="%1$s">' .
					esc_html__( 'WordPress Dashboard', 'livicons-evo' ) .
					'</a>.',
					esc_url( admin_url() )
				)
			);
		}
	}
}
add_action( 'admin_init', 'livicons_evolution_check_mandatory_params' );

// makes available for translation
function livicons_evolution_textdomain_init() {
	load_plugin_textdomain( 'livicons-evo', false, dirname( LIVICONS_EVOLUTION_PLUGIN_SLUG ) . '/languages' );
}
add_action( 'init', 'livicons_evolution_textdomain_init' );

// factory defaults
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-defaults.php' );

// full icons list
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-icons-list.php' );

// validation and sanitization
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-validation.php' );

// styles and scripts
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-assets.php' );

// admin part of the plugin
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-admin.php' );

// dialog box for customizing
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-dialog.php' );

// shortcode
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-shortcode.php' );

// WP Bakery page builder (visual composer) add-on
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-visual-composer.php' );

// Elementor plugin add-on
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-elementor.php' );

// automatic updates
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-update.php' );

// activation of the plugin
require_once( LIVICONS_EVOLUTION_PATH . 'includes/livicons-evolution-activation.php' );

?>